<?php

declare(strict_types=1);

namespace Drupal\Tests\pathologic\Kernel;

use Drupal\Component\Utility\Html;
use Drupal\Core\Url;
use Drupal\filter\Entity\FilterFormat;
use Drupal\KernelTests\KernelTestBase;

/**
 * Base class for all Pathologic Kernel tests.
 */
abstract class PathologicKernelTestBase extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'filter',
    'pathologic',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * The ID of a text format to be used in a test.
   *
   * @var formatId
   */
  protected $formatId = '';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installConfig(['system', 'filter', 'pathologic']);
  }

  /**
   * Build a text format with Pathologic configured a certain way.
   *
   * @param array $settings
   *   An array of settings for the Pathologic instance on the format.
   *
   * @return string
   *   The randomly generated format machine name for the new format.
   */
  protected function buildFormat(array $settings) {
    $this->formatId = ($settings['local_settings']['protocol_style'] ?? 'unknown') . '_' . $this->randomMachineName(8);
    $format = FilterFormat::create([
      'format' => $this->formatId,
      'name' => $this->formatId,
    ]);
    $format->setFilterConfig('filter_pathologic', [
      'status' => 1,
      'settings' => $settings,
    ]);
    $format->save();
    return $this->formatId;
  }

  /**
   * Runs the given string through the Pathologic text filter.
   *
   * @param string $markup
   *   Raw markup to be processed.
   *
   * @return string
   *   A string of text-format-filtered markup.
   */
  protected function runFilter(string $markup): string {
    return check_markup($markup, $this->formatId)->__toString();
  }

  /**
   * Wrapper around url() which does HTML entity decoding and encoding.
   *
   * Since Pathologic works with paths in content, it needs to decode paths that
   * have been HTML-encoded, and re-encode them when done. This is a wrapper
   * around url() which does the same thing so that we can expect the results
   * from it and from Pathologic to still match in our tests.
   *
   * @see url()
   * @see http://drupal.org/node/1672932
   * @see http://www.w3.org/TR/xhtml1/guidelines.html#C_12
   */
  protected function pathologicContentUrl($path, $options) {
    // If we pretend this is a path to a file, make url() behave like clean
    // URLs are enabled.
    // @see _pathologic_replace()
    // @see http://drupal.org/node/1672430
    if (!empty($options['is_file'])) {
      $options['script_path'] = '';
    }
    if (parse_url($path, PHP_URL_SCHEME) === NULL) {
      if ($path == '<front>') {
        return Html::escape(Url::fromRoute('<front>', [], $options)->toString());
      }
      $path = 'base://' . $path;
    }
    return Html::escape(Url::fromUri(htmlspecialchars_decode($path, ENT_COMPAT), $options)->toString());
  }

}
